#!/usr/bin/env python
# -*- coding: utf-8 -*-
from tvm.api.io import err

import os
import platform
import subprocess
import threading


class ExecuteWithTimeout(threading.Thread):
    def __init__(self, cmd, timeout):
        threading.Thread.__init__(self)
        self.cmd = cmd
        self.timeout = timeout

    def run(self):
        self.p = subprocess.Popen(self.cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        self.p.wait()

    def execute(self):
        self.start()
        self.join(self.timeout)

        if self.is_alive():
            self.p.terminate()  # self.p.kill() if need.
            self.join()


class Notification(object):
    def __init__(self):
        self.app_path = os.path.join(os.path.dirname(__file__), '../vendor/MacOS/TVManager.app')
        self.bin_path = os.path.join(self.app_path, 'Contents/MacOS/TVManager')
        self.bin_path = os.path.abspath(self.bin_path)

        if not self.is_available():
            raise Exception('Mac OS notifications supported on Mac OS X 10.8, or higher.')

        if not os.access(self.bin_path, os.X_OK):
            os.chmod(self.bin_path, 111)
            if not os.access(self.bin_path, os.X_OK):
                raise Exception('You have no privileges to execute \'%s\'' % self.bin_path)

    def notify(self, message, **kwargs):
        args = ['-message', message]
        args += [a for b in [('-%s' % arg, str(key)) for arg, key in kwargs.items()] for a in b]  # flatten list

        ExecuteWithTimeout([self.bin_path, ] + args, 3).execute()

    @staticmethod
    def is_available():
        """ Returns whether or not the current platform is Mac OS X 10.8, or higher. """
        return platform.system() == 'Darwin' and platform.mac_ver()[0] >= '10.8'


class EmptyNotification:
    def __init__(self, error):
        self.error = error

    def notify(self, *args, **kwargs):
        err(self.error)

try:
    Notifier = Notification()
except Exception, error:
    Notifier = EmptyNotification(error)
