#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Proxy over ffprobe to retrieve data about episode video files. """
import subprocess
import json
import shlex
import os
import datetime

from tvm.api.io import open_with_default


class VideoSource(object):
    @staticmethod
    def all(episode_path):
        if not episode_path or not os.path.isfile(episode_path):
            return {}

        ffprobe_args = 'ffprobe -v quiet -print_format json -show_format -show_streams'
        command = shlex.split(ffprobe_args) + [episode_path]
        process = subprocess.Popen(command,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)

        stdout, _ = process.communicate()
        return json.loads(stdout)

    @staticmethod
    def is_video(episode_path):
        return bool(VideoSource.all(episode_path))

    @staticmethod
    def duration(episode_path):
        duration = VideoSource.all(episode_path)['format']['duration']
        return datetime.timedelta(seconds=float(duration))

    @staticmethod
    def open(episode_path):
        return open_with_default(episode_path) if episode_path else False
