#!/usr/bin/env python
# -*- coding: utf-8 -*-
from datetime import datetime

import os
import json

from tvm.api.config import config
from tvm.api.io import info


class RSSCache(object):
    @staticmethod
    def cache(rss):
        if not os.path.exists(config.cache_path):
            os.makedirs(config.cache_path)

        now = datetime.now()
        info('Caching cleansed RSS response to: "%s.json"' % os.path.join(config.cache_path, str(now)))
        with open(os.path.join(config.cache_path, str(now) + '.json'), 'w') as cached_file:
            json.dump(rss.cleansed_updates, cached_file)

    @staticmethod
    def load():
        if not os.path.exists(config.cache_path):
            return []

        cached_objects = []
        for cached_file in os.listdir(config.cache_path):
            if cached_file.startswith('.'):
                continue

            with open(os.path.join(config.cache_path, cached_file), 'r') as cached:
                cached_objects.append(json.load(cached))

        return cached_objects
