#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from collections import defaultdict

from tvm.commands import base

from tvm.api.config import config
from tvm.api.io import print_errors
from tvm.api.io import err, ok


class DoctorCommand(base.BaseCommand):
    def __init__(self, **arguments):
        super(self.__class__, self).__init__(**dict(arguments, **{'_terminate_program': False}))

        self.helper = DoctorHelper(self.collection)
        self._extact_args(arguments)
        self._exec()

    def _extact_args(self, arguments):
        self._records = arguments.get('--records', False)
        self._repository = arguments.get('--repository', False)

    def _exec(self):
        if self.collection.load_success:
            ok('Semantic check on recorded files successfully completed.')

        if self._records:
            raw_input('\nPress enter to validate_exists_in_records...')
            self.helper.validate_exists_in_records()

        if self._repository:
            raw_input('\nPress enter to validate_exists_in_repository...')
            self.helper.validate_exists_in_repository()


class DoctorHelper(base.BaseHelper):
    def __init__(self, collection):
        super(self.__class__, self).__init__(collection)

    def _load_stored_files(self):
        stored_shows, errors = {}, defaultdict(list)
        for path in config.physical_storage_paths:
            if not os.path.exists(path):
                err('Failed to access path "%s".' % path)
                continue

            ok('Accessing path, "%s".' % path)

            # Walk all defined paths to get all shows stored.
            for root, _, files in os.walk(path):
                for episode in files:
                    episode_path = os.path.join(root, episode)
                    key = self.collection.episode_path_to_record(episode_path)

                    episode = os.path.splitext(episode_path)[0]
                    if config.ignore_pattern.match(os.path.basename(episode)):
                        continue

                    if key in stored_shows:
                        errors['duplicate physical files'].append(episode_path)

                    stored_shows[key] = episode_path

        print_errors(errors, terminate_program=False)
        return stored_shows

    def validate_exists_in_records(self):
        errors = defaultdict(list)
        for stored_show, path in self._load_stored_files().iteritems():
            if stored_show not in self.collection.collection:
                errors['stored and not recorded'].append(path)

        if not errors:
            ok('validate_exists_in_records completed.')
        print_errors(errors)

    def validate_exists_in_repository(self):
        errors = defaultdict(list)
        repository = self._load_stored_files().keys()

        for entry in self.collection.collection.keys():
            if entry not in repository:
                errors['recorded and not stored'].append(entry)

        if not errors:
            ok('validate_exists_in_repository completed.')
        print_errors(errors)
