#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from tvm.commands import base
from tvm.api.io import err, info, info2
from tvm.api.dates import c_datetime


class GenerateCommand(base.BaseCommand):
    def __init__(self, **arguments):
        super(self.__class__, self).__init__(**dict(arguments, **{'_ignore_errors': True}))

        self.helper = GenerateHelper(self.collection)
        self._extact_args(arguments)
        self._exec()

    def _extact_args(self, arguments):
        self._path = os.path.abspath(os.path.expanduser(arguments['<path>']))
        if not os.path.exists(self._path) or not os.path.isdir(self._path):
            err('<path> must be an existing directory.')
            exit(0)

        if not os.listdir(self._path):
            info('No potential records were found in "%s".' % self._path)
            exit(0)

        self._append = arguments.get('--append', False)
        self._force = arguments.get('--force', False)

    def _exec(self):
        records = self.helper.load_stored_files(self._path, force=self._force)
        if not records:
            return info('No new records were found.')

        self.helper.print_records(records)
        if not self._append:
            return info('%d new records were found. `--append` to add to records.' % len(records))

        self.collection.save(records, force=self._force)  # force=True to force write to disk.
        info('New entries recorded: %d (%d total).' % (len(records),
                                                       len(self.collection.collection) + len(records)))


class GenerateHelper(base.BaseHelper):
    def __init__(self, collection):
        super(self.__class__, self).__init__(collection)

    def load_stored_files(self, path, force=False):
        """ Set `force_add=True` to include a found entry even if it has already been recorded. """
        new_records = []
        for dirname, _, filenames in os.walk(path):
            for filename in filenames:
                if filename.startswith('.'):
                    continue

                episode_path = os.path.join(dirname, filename)
                key = self.collection.episode_path_to_record(episode_path)

                # Don't add it if it has already been recorded.
                if key in self.collection.collection and not force:
                    continue

                new_records.append((c_datetime(episode_path), ) + key)

        return new_records

    def print_records(self, records):
        info('These are the records generated:')
        for record in records:
            info2(self.collection.record_to_csv([record]))
